--*******************************************************************************************************************************
--
-- Description: This module throttles data flow based on an external enable or a requested sample count.
--					
-- Rev History: 1-10-2014:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

use work.model_specific_package.all;
use work.transceiver_package.all;
use work.tb_package.all;

entity tx_dma_emulator is
port (		
    -- Clock and reset.
    user_clk		: in std_logic;
    user_reset		: in std_logic;
    
    -- Transmit signals
    tx_buff_full	: out std_logic;
    
	-- Direct memory access (dma) interface connections to the application logic.
	dma_mem_wr 		: out std_logic;
	dma_wrchannel 	: out std_logic;
	dma_status		: out std_logic;
	dma_control		: in std_logic_vector(5 downto 0);
	dma_wrdata		: out std_logic_vector(AXI_WIDTH-1 downto 0)
	);
end tx_dma_emulator;   

architecture structural of tx_dma_emulator is
	-- Array of #of_bursts by burst_size    
	type TX_DATA is array (1 to (BURST_SIZE/16)*BURST_COUNT) of std_logic_vector(AXI_WIDTH-1 downto 0); -- Array containing AXI_WIDTH-bit DMA buffer samples. The burst size has been divided by 16 because there are 16 bursts per line.
	        
    -- Data signals
	signal txdata	  : TX_DATA;     
	signal synth_dacdata : std_logic_vector(AXI_WIDTH-1 downto 0);
	-- Control Signals
	signal j		: integer; 
	signal k		: integer; 
	signal ZERO		: std_logic_vector(63 downto 0);
    signal ONE		: std_logic_vector(63 downto 0);
    
    type pciestate is (pcie_load_data, pcie_idle,  pcie_burst_wait, pcie_burst_start, pcie_burst_complete);	-- Type definitions for the PCI Express state machine.
	signal pcie_state		: pciestate;  					-- Current PCI Express machine state.

                         
begin                                                          
	ZERO <=(others=>'0');
	ONE  <=(others=>'1');
       

	SYNTH_DAC_DATA : entity work.rx_counter 
		port map(
    		clk				=> user_clk,
    		rst				=> user_reset,     
    		test_rddata		=> synth_dacdata,
    		count_data		=> open 
    	);           

	--************************************************* STATE MACHINE TRANSITIONS **************************************************
	PCI_EXPRESS_STATE_MACHINE : process (user_clk, user_reset)
	begin
		if user_reset = '1' then
		   	pcie_state <= pcie_idle;
		   	
		elsif rising_edge(user_clk) then
			case pcie_state is
				when pcie_idle =>
					-- Wait for dma_control to be set to start dma transfer. 
					pcie_state <= pcie_load_data;

				when pcie_load_data => 
					-- Wait while a dma buffer fills with sythetic DAC data.
					if j = (BURST_SIZE/16)*BURST_COUNT then
						pcie_state <= pcie_burst_wait;
					else 
						pcie_state <= pcie_load_data;						
					end if;
					
				when pcie_burst_wait =>
					-- Wait for dma_control to be set to start dma transfer. 
					if dma_control(4 downto 0) /= "00000" then
						pcie_state <= pcie_burst_start;
					end if;
										
				when pcie_burst_start =>
					-- Wait till one burst of data has been transfered.
					if j = (BURST_SIZE/16)*k then
						pcie_state <= pcie_burst_complete;
					else 
						pcie_state <= pcie_burst_start;						
					end if;
					
				when pcie_burst_complete =>
					pcie_state <= pcie_burst_wait;
					
					
				-- Define the default state to be dma_idle.
				when others => pcie_state <= pcie_idle;
			end case;
		end if;
	end process;
	
	--*************************************************** STATE MACHINE EVENTS *****************************************************
	process (user_clk,user_reset)
	begin
		if user_reset = '1' then
			j <= 1;
			k <= 1;
			tx_buff_full <= '0';
			dma_wrchannel <= '0';	-- After you get request send back grant to send data                                         
			dma_mem_wr <= '0';	-- Send rd_en to dma_fifo   
		   	dma_status <= '0';
		   	
		elsif rising_edge(user_clk) then

			-- Save incoming read data to memory array.
			if pcie_state = pcie_load_data then
				tx_buff_full <= '0';
				txdata(j) <= synth_dacdata;
				if j = BURST_SIZE/16 and k < BURST_COUNT then  
					j <= 1;
					k <= k + 1;
				elsif j = BURST_SIZE/16 and k = BURST_COUNT then
					j <= 1;                             
					k <= 1;
				else
					j <= j + 1;
					k <= k;
				end if;    
			end if;                       
			
			-- Make sure all status bits start and end at 0.
			if pcie_state = pcie_burst_wait then
				tx_buff_full <= '1';
				j <= 1; -- Reset the location of data so when data starts to transmit it starts with the first address.
				k <= 1; -- Reset the location of data so when data starts to transmit it starts with the first burst.
				dma_status <= '0';
				dma_wrchannel <= '0';                                          
				dma_mem_wr <= '0';   			
			end if; 
				
			if pcie_state = pcie_burst_start then
				dma_wrdata <= txdata(j);
				j <= j + 1;
				
				dma_status <= '1';
				dma_wrchannel <= '1'; -- After you get request send back grant to send data                                         
				dma_mem_wr <= '1'; -- Send wr_en to dma_fifo   
			end if;
			
			-- Set control signals high only after a request is received and until all data is logged.
			if pcie_state = pcie_burst_complete then
				dma_status <= '0';
				dma_wrchannel <= '0'; -- After you get request send back grant to send data                                         
				dma_mem_wr <= '0'; -- Send wr_en to dma_fifo   
			end if;
			
		end if;
	end process; 
	          	
end structural;

