--*******************************************************************************************************************************
--
-- Description: This module acquires data and over-range status from a variety of A/D converter(ADC) chips.  The bus type
--				supports multiple RX channels and stuffing of multiple samples per clock cyle to accomodate DDR inputs.  When
--				samples are stuffed, they are chronologically ordered from LSB to MSB.
--					
-- Rev History: 09-04-2013 Initial Release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all; 

Library UNISIM; 
use UNISIM.Vcomponents.all;

use work.model_specific_package.all;
use work.transceiver_package.all;

entity adc_interface is 
	port (
	--  Reset and clocks supplied by the application logic.
	rst         	: in std_logic;		-- Active high asynchronous reset.
	clk 			: in std_logic;		-- Bus clock.
	rx_clk			: in std_logic;		-- Clock used to register ADC sample data.
	clk200 			: in std_logic;		-- 200 MHz clock on global clock buffer.
	
	-- ADC data supplied to the application logic.
	rx_data_out		: out RX_BUS;		-- Data bus carrying ADC samples.
	rx_ovr_out		: out OVR_BIT;		-- Logic one indicates an over-range condition on the ADC.

	-- Status of the IDELAYCTRL primitive.
	idelay_rdy		: out std_logic; 	-- Logic one indicates that the IDELAYCTRL primitive is stable.
	
	-- Serial clock lock command signal supplied from/to the application logic.
	clk_lck_start	: in std_logic; 	-- Single period enable to initiate the clock lock process.
	clk_lck_done	: out std_logic; 	-- Single period flag to indicate that the clock lock process is complete.
	
	--	The following signals connect to ADC devices through FPGA pins.
	rx_data_in		: in RX_BUS_IN;		-- ADC data bus.
	rx_ovr_in		: in RX_1;			-- ADC over-range flag.
	rx_clk_in		: in RX_1			-- ADC output clock.
);	
end adc_interface;

architecture behavioral of adc_interface is

	type OCT_CHU is array (3 downto 0) of unsigned(11 downto 0);
	signal source		: OCT_CHU;
	
begin
 	
--****************************************************** BINARY COUNTER *******************************************************

	-- Generate a binary count on both channels (0 to 2047).
	process (rx_clk, rst)
    begin
		if rst='1' then
			for j in 0 to 3 loop
				source(j) <= to_unsigned(j,12);
			end loop;
        elsif rising_edge(rx_clk) then
			for j in 0 to 3 loop
				source(j) <= '0' & (source(j)(10 downto 0) + to_unsigned(4,11));
			end loop;
		end if;
	end process;
	
    rx_data_out(1) <= std_logic_vector(source(3)) & std_logic_vector(source(2)) & std_logic_vector(source(1)) & std_logic_vector(source(0));
    rx_data_out(2) <= std_logic_vector(source(3)) & std_logic_vector(source(2)) & std_logic_vector(source(1)) & std_logic_vector(source(0));
    rx_ovr_out(1) <= x"0";
    rx_ovr_out(2) <= x"0";
    
    idelay_rdy <= '1';
    clk_lck_done <= clk_lck_start;
	
end behavioral;
