--*******************************************************************************************************************************
--
-- Description: This module performs all PCI Express write/read operations to base address register (BAR) two.
--				The AXI_WIDTH constant determines whether slave_dataout is 64-bit or 128-bit aligned.
--					
-- Rev History: 05-21-2014:	Consolidated QDR II+ SRAM registers and added control signal for the core calibration function.
--				09-04-2013:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.model_specific_package.all;
use work.transceiver_package.all;
use work.dma_package.all;

entity bar2_decoder is
port (
    -- Reset and clock.                                                                                      
	rst    				: in std_logic;									-- Active high asynchronous reset.
	clk		        	: in std_logic;									-- Bus clock.
	                                                                                                                           
    -- PCI Express write transaction interface.
	slv_addr 			: in std_logic_vector(15 downto 2);				-- Refer to the PLDA EZDMA2 reference manual.
    slv_dataout     	: in std_logic_vector(AXI_WIDTH-1 downto 0);	-- Refer to the PLDA EZDMA2 reference manual.
	slv_bytevalid 		: in std_logic_vector((AXI_WIDTH/8)-1 downto 0);-- Refer to the PLDA EZDMA2 reference manual.
	bar2_write 			: in std_logic;									-- Logic one indicates that the current PCI write request is directed at BAR2.
                                                                                                                              
    -- PCI Express read transaction interface.
	bar2_read			: in std_logic;									-- Logic one indicates that the current PCI read request is directed at BAR2.
	bar2_rddata			: out std_logic_vector(AXI_WIDTH-1 downto 0);	-- Data return from read operation on BAR2.
    dma_rd          	: in std_logic;									-- Refer to the PLDA EZDMA2 reference manual.
    dma_rdaddr      	: in std_logic_vector(15 downto 3);				-- Refer to the PLDA EZDMA2 reference manual.
	dma_rdchannel 		: in  std_logic;								-- Refer to the PLDA EZDMA2 reference manual.
	                                                                                                                       	       
	-- QDR II+ SRAM control and status signals.
	qdr_wr_data			: out std_logic_vector(71 downto 0);			-- Data bus carrying four 18-bit QDR II+ SRAM write data values.
	qdr_bw_n			: out std_logic_vector(7 downto 0);				-- Eight byte enables assigned to the QDR II+ SRAM write data values.
	qdr_rd_data			: in std_logic_vector(71 downto 0);				-- Data bus carrying four 18-bit QDR II+ SRAM read data values.
	qdr_addr			: out std_logic_vector(QDR_ADDR_WIDTH-1 downto 0);	-- Address selected for current operation.
	qdr_cmd				: out DUAL_2;									-- Single period write (0) and read (1) command strobes in the PCI clock domain.
	qdr_rd_valid		: in std_logic;									-- Logic one indicates that read data is available from the QDR II+ SRAM.
 	qdr_calib			: out std_logic;								-- Logic one initiates calibration of the QDR II+ SRAM.
    
	-- PCI performance monitor.
 	pci_measurement		: out std_logic_vector(1 downto 0);				-- Reset the PCI performance measurement counters.
	pci_throughput		: in std_logic_vector(63 downto 0);				-- Report the measured PCI DMA throughput in bytes/sec (transmitter=63:32, receiver=31:0).
	pci_latency			: in std_logic_vector(63 downto 0);				-- Report the maximum PCI DMA latency in 4 ns clock periods (transmitter=63:32, receiver=31:0).

    -- Command and status signals for each channel.
	synchronizer		: out CH_TYPE_36;								-- Register containing start and stop event selections.
	synchronizer_on_off	: out CH_TYPE_2;								-- Active high on/off (bit 0/bit 1) switch for the synchronizer.
	synchronizer_status	: in CH_TYPE_2;									-- Indicator bits for busy (0) and ready (1) status reporting.
	
	-- Command, status, and error signals.
	soft_trig			: out std_logic;								-- Software trigger requested over the PCIe bus.
	led_control 		: out std_logic_vector(1 downto 0);				-- Logic one illuminates each of the two LEDs.
	channel_monitor 	: in CH_TYPE_3; 								-- Bus used to convey DMA refresh status and channel errors.
	channel_status 		: out CH_TYPE_3;  								-- Bus used to convey channel status since last queried.
	channel_int_mask 	: out CH_TYPE_3;  								-- Mask bits to suppress interrupts based on channel status updates.
	bar2_address_error 	: out std_logic_vector(1 downto 0)				-- Bus used to convey a BAR2 PCI read or write error.
	
);                                                                                                                     	
end bar2_decoder;                                                                                                      	   

architecture structural of bar2_decoder is                                                                             	
                                                                                                                               	
	signal bar2_slave_read			: std_logic;						-- Logic one indicates an active slave read to BAR2 address space.
	signal bytevalid				: std_logic_vector(1 downto 0);		-- Logic one indicates that all PCI byte valids are active.
	signal dma_rdaddr_hold			: std_logic_vector(3 downto 0);		-- Store the read address to select the correct rddata bus.
	signal address_error_common		: std_logic_vector(1 downto 0);		-- BAR2 address errors that occur within the base range.
	signal address_error_channel	: CH_TYPE_2;						-- BAR2 address errors that occur within a channel range.
	signal address_error_ch			: DUAL_CHU;							-- BAR2 address errors that occur within a channel range.
	signal bar2_rddata_common		: std_logic_vector(AXI_WIDTH-1 downto 0);-- Data return from a read operation within the BAR2 common range.
	signal bar2_rddata_channel		: CH_TYPE_AXI;						-- Data return from a read operation within the BAR2 channel range.
	signal bar2_rddata_ch			: DMA_AXI;						-- Data return from a read operation within the BAR2 channel range.

begin

--************************************************** DECODE READ TRANSACTIONS **************************************************

    -- Qualify an active read transaction to BAR2.        
	bar2_slave_read <= dma_rdchannel and dma_rd and bar2_read;
	
	-- Create an active high enable for write operations to the upper 32 register bits.
	bytevalid(1) <= (slv_bytevalid(7) and slv_bytevalid(6) and slv_bytevalid(5) and slv_bytevalid(4)) when slv_addr(3) = '0'
		else (slv_bytevalid((AXI_WIDTH/8)-1) and slv_bytevalid((AXI_WIDTH/8)-2) and slv_bytevalid((AXI_WIDTH/8)-3) and slv_bytevalid((AXI_WIDTH/8)-4));
	-- Create an active high enable for write operations to the lower 32 register bits.
	bytevalid(0) <= slv_bytevalid(3) and slv_bytevalid(2) and slv_bytevalid(1) and slv_bytevalid(0) when slv_addr(3) = '0'
		else slv_bytevalid((AXI_WIDTH/8)-5) and slv_bytevalid((AXI_WIDTH/8)-6) and slv_bytevalid((AXI_WIDTH/8)-7) and slv_bytevalid((AXI_WIDTH/8)-8);
        
    process (clk,rst)
    begin
        if rst = '1' then  
			dma_rdaddr_hold <= (others => '0');
			bar2_rddata <= (others => '0');
			
        elsif rising_edge (clk) then
  
        	-- Register the top byte of the read address to determine which bar2_rddata bus is active (see below).
            if bar2_slave_read = '1' then
				dma_rdaddr_hold <= dma_rdaddr(15 downto 12);
            end if;
            
			--  Select the DMA read data bus based on the address requested by the PLDA EZDMA2 controller.
			case dma_rdaddr_hold is
				when x"0" => bar2_rddata <= bar2_rddata_common;
				when x"1" => bar2_rddata <= bar2_rddata_ch(1);
				when x"2" => bar2_rddata <= bar2_rddata_ch(2);
				when x"3" => bar2_rddata <= bar2_rddata_ch(3);
				when x"4" => bar2_rddata <= bar2_rddata_ch(4);
				when x"5" => bar2_rddata <= bar2_rddata_ch(5);
				when x"6" => bar2_rddata <= bar2_rddata_ch(6);
				when x"7" => bar2_rddata <= bar2_rddata_ch(7);
				when x"8" => bar2_rddata <= bar2_rddata_ch(8);
				when others => bar2_rddata <= bar2_rddata_common;
			end case;
			
        end if;
    end process;
    
	-- Assign a slave read datapath to all four possible DMA channels.
	BAR2_RD : for i in 1 to 8 generate
		-- Assign a slave read datapath to each active channel.
		RD_ACTIVE: if i < TX_CHANNEL_COUNT+RX_CHANNEL_COUNT+1 generate
			bar2_rddata_ch(i) <= bar2_rddata_channel(i);
		end generate;	
		-- Make signal assignments to unused channels.
		RD_UNUSED: if i > TX_CHANNEL_COUNT+RX_CHANNEL_COUNT generate
			bar2_rddata_ch(i) <= (others => '0');
		end generate;	
	end generate;

--******************************************************* COMMON DECODER *******************************************************

    -- BAR2 address decoder for common registers that are not assigned to a specific channel.
    COMMON : entity work.bar2_common
	port map (
		rst						=> rst,
		clk						=> clk,
	    slv_addr 				=> slv_addr,
	    slv_dataout       		=> slv_dataout,
		bytevalid 	  			=> bytevalid,
		bar2_write 		  		=> bar2_write,
		bar2_slave_read		  	=> bar2_slave_read,
		bar2_rddata_common		=> bar2_rddata_common,
	    dma_rdaddr        		=> dma_rdaddr,
		qdr_wr_data				=> qdr_wr_data,
		qdr_bw_n				=> qdr_bw_n,
		qdr_rd_data				=> qdr_rd_data,
		qdr_addr				=> qdr_addr,
		qdr_cmd					=> qdr_cmd,
		qdr_rd_valid			=> qdr_rd_valid,
		qdr_calib				=> qdr_calib,
		pci_measurement			=> pci_measurement,
		pci_throughput			=> pci_throughput,
		pci_latency				=> pci_latency,
		soft_trig				=> soft_trig,
		led_control				=> led_control,
		address_error			=> address_error_common
	);
	
--***************************************************** RX CHANNEL DECODER *****************************************************

	-- Generate a BAR2 address decoder for each RX channel.
	RX_CH: if RX_CHANNEL_COUNT > 0 generate
		-- Address decoder generated with a base address referenced to the RX channel number.
		RX_NUM : for i in 1 to zero_to_one(RX_CHANNEL_COUNT) generate
			RX_CH : entity work.bar2_channel
			generic map (
				CHANNEL_NUMBER			=> i
			)
			port map (
				rst						=> rst,
				clk						=> clk,
				slv_addr 				=> slv_addr,
				slv_dataout       		=> slv_dataout,
				bytevalid 	  			=> bytevalid,
				bar2_write 		  		=> bar2_write,
				bar2_slave_read		  	=> bar2_slave_read,
				bar2_rddata_channel		=> bar2_rddata_channel(i),
				dma_rdaddr        		=> dma_rdaddr,
				synchronizer			=> synchronizer(i),
				synchronizer_on_off		=> synchronizer_on_off(i),
				synchronizer_status		=> synchronizer_status(i),
				channel_monitor			=> channel_monitor(i),
				channel_status			=> channel_status(i),
				channel_int_mask		=> channel_int_mask(i),
				address_error			=> address_error_channel(i)
			);
		end generate;
	end generate;
	
--***************************************************** TX CHANNEL DECODER *****************************************************

	-- Generate a BAR2 address decoder for each TX channel.
	TX_CH: if TX_CHANNEL_COUNT > 0 generate
		-- Address decoder generated with a base address referenced to the TX channel number.
		TX_NUM : for i in RX_CHANNEL_COUNT+1 to RX_CHANNEL_COUNT+TX_CHANNEL_COUNT generate
			TX_CH : entity work.bar2_channel
			generic map (
				CHANNEL_NUMBER			=> i
			)
			port map (
				rst						=> rst,
				clk						=> clk,
				slv_addr 				=> slv_addr,
				slv_dataout       		=> slv_dataout,
				bytevalid 	  			=> bytevalid,
				bar2_write 		  		=> bar2_write,
				bar2_slave_read		  	=> bar2_slave_read,
				bar2_rddata_channel		=> bar2_rddata_channel(i),
				dma_rdaddr        		=> dma_rdaddr,
				synchronizer			=> synchronizer(i),
				synchronizer_on_off		=> synchronizer_on_off(i),
				synchronizer_status		=> synchronizer_status(i),
				channel_monitor			=> channel_monitor(i),
				channel_status			=> channel_status(i),
				channel_int_mask		=> channel_int_mask(i),
				address_error			=> address_error_channel(i)
			);
		end generate;
	end generate;
    
--************************************************** ERROR SIGNAL ASSIGNMENTS **************************************************

	-- Map individual address error channel bits into a single bus so that they can be logically ORed (see below).
	BAR2_ERROR : for i in 1 to TX_CHANNEL_COUNT+RX_CHANNEL_COUNT generate
		address_error_ch(0)(i) <= address_error_channel(i)(0);
		address_error_ch(1)(i) <= address_error_channel(i)(1);
	end generate;	
		
	-- Generate an error if an invalid BAR2 address is reported by any of the decoders.
	bar2_address_error(0) <= '1' when address_error_common(0) = '1' or to_integer(address_error_ch(0)) /= 0 or (bar2_write = '1' and to_integer(unsigned(slv_addr(15 downto 12))) > TX_CHANNEL_COUNT+RX_CHANNEL_COUNT) else '0';
	bar2_address_error(1) <= '1' when address_error_common(1) = '1' or to_integer(address_error_ch(1)) /= 0 or (bar2_slave_read = '1' and to_integer(unsigned(dma_rdaddr(15 downto 12))) > TX_CHANNEL_COUNT+RX_CHANNEL_COUNT) else '0';
	
end structural;
