--*******************************************************************************************************************************
--
-- Description: This module is used to assign source clocks to global buffers and connect them to desired inputs.
--
-- Rev History: 09-04-2013
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

Library UNISIM; 
use UNISIM.Vcomponents.all; 

use work.model_specific_package.all;
use work.transceiver_package.all;

entity clock_route is 
	port (
	-- Reset and clocks.
	rst         	: in std_logic;		-- Active high asynchronous reset.       
	sample_clk_in	: in std_logic;		-- IOB sample clock.
	rx_clk_in		: in RX_1;			-- IOB ADC forwarded clock.
	tx_clk_in		: in TX_1;			-- IOB DAC forwarded clock.
    clk200_in		: in std_logic;		-- 200 MHz clock from an external oscillator.
    clk500_in		: in std_logic;		-- 500 MHz clock from an external oscillator.
	
	-- Output clocks.
	rx_clk			: out std_logic;	-- Clock assigned to the receiver datapath.
	tx_clk			: out std_logic;	-- Clock assigned to the transmitter datapath.
    clk200			: out std_logic;	-- 200 MHz clock on a global clock buffer.
    clk500			: out std_logic		-- 500 MHz clock on a global clock buffer.
	);	
	end clock_route;	

architecture behavioral of clock_route is

	signal sample_clk	: std_logic;	-- Sample clock on global clock buffer.

begin                    

--**************************************************** GLOBAL CLOCK BUFFERS ****************************************************

	-- Assign the 200 MHz oscillator source to a global clock buffer.
	CLK200_BUFG : BUFG port map (I => clk200_in, O => clk200);
	
	-- Assign the 500 MHz oscillator source to a global clock buffer.
	CLK500_BUFG : BUFG port map (I => clk500_in, O => clk500);
	
	-- Assign the sample clock source to a global clock buffer.
	SAMPLE_CLK_BUFG : BUFG port map (I => sample_clk_in, O => sample_clk);
		
--************************************************* RECEIVER CLOCK ASSIGNMENTS *************************************************

	-- Connect the receiver clock to sample clock if requested.
	RX_CLK_SAMPLE: if RX_CHANNEL_COUNT > 0 and RX_CLK_SELECT = 0 generate
		rx_clk <= sample_clk;
	end generate;
	
	-- Tie the receiver clock low if there are no receiver channels.
	RX_CLK_NONE: if RX_CHANNEL_COUNT = 0 generate
		rx_clk <= '0';
	end generate;
	
--*********************************************** TRANSMITTER CLOCK ASSIGNMENTS ************************************************

	-- Connect the transmitter clock to sample clock if requested.
	TX_CLK_SAMPLE: if TX_CHANNEL_COUNT > 0 and TX_CLK_SELECT = 0 generate
		tx_clk <= sample_clk;
	end generate;
	
	-- Connect the transmitter clock to the DAC clock if requested.
	TX_CLK_DAC: if TX_CHANNEL_COUNT > 0 and TX_CLK_SELECT = 1 generate
		DAC_CLK_BUFG : BUFG port map (I => tx_clk_in(RX_CHANNEL_COUNT+1), O => tx_clk);
	end generate;
	
	-- Tie the transmitter clock low if there are no transmitter channels.
	TX_CLK_NONE: if TX_CHANNEL_COUNT = 0 generate
		tx_clk <= '0';
	end generate;
	

end behavioral;
