--*******************************************************************************************************************************
--
-- Description: This module contains all of the recurring modules needed to create a receiver channel.
--					
-- Rev History: 09-04-2013:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

use work.model_specific_package.all;
use work.transceiver_package.all;

entity rx_datapath is
port (
    -- Reset and clocks.
	rst 				: in std_logic;							-- Active high asynchronous reset.
	user_clk 			: in std_logic;							-- System clock.
	rx_clk 				: in std_logic;							-- Receiver clock.

    -- Input to the datapath.
	data_item			: in std_logic_vector((RX_ITEM_STUFF*RX_ITEM_SIZE)-1 downto 0);	-- Input data items that may be stuffed.	
	
    -- Synchronizer command and status.
	synchronizer_on_off	: in std_logic_vector(1 downto 0);		-- Active high on/off (bit 0/bit 1) switch for the synchronizer.
	synchronizer_status	: out std_logic_vector(1 downto 0);		-- Indicator bits for busy (0) and ready (1) status reporting.
	start_event			: in std_logic_vector(1 downto 0);		-- Select the event to activate the datapath after it is enabled. (none, software trigger, hardware trigger)
	stop_event			: in std_logic_vector(1 downto 0);		-- Select the event to deactivate the datapath. (software command, software trigger, hardware trigger, sample count)
	sample_count		: in std_logic_vector(31 downto 0);		-- Number of samples to collect when sample count is selected as the stop event.
	soft_trigger		: in std_logic;							-- Single period software trigger to be used by the application logic.
	hard_trigger		: in std_logic;							-- Hardware rising edge start trigger and falling edge stop trigger.

	-- PCI performance command.
 	pci_measurement		: in std_logic;							-- Enable continuous DMA on the channel.

	-- Connections to the DMA control and status registers in BAR0.
	dma_page_address	: in std_logic_vector(63 downto 0);		-- PCI address of the current DMA page in host memory.
	dma_descriptor		: in std_logic_vector(31 downto 0);		-- Descriptor that defines the parameters of the current DMA transaction.
    dma_current_page	: out std_logic_vector(9 downto 0);		-- Page number of the active DMA buffer.
    dma_current_burst	: out std_logic_vector(15 downto 0);	-- Number of the active DMA burst within a page.
	dma_rddata			: out std_logic_vector(AXI_WIDTH-1 downto 0);	-- DMA local data bus used for write transactions.
	dma_marker			: out std_logic;						-- Logic one indicates that the DMA channel has been refreshed.
	fifo_error			: out std_logic;						-- Logic one indicates a FIFO overflow condition.
	
	-- Direct memory access (dma) interface connections to the PLDA EZDMA2 core.
	dma_wr				: in std_logic;							-- Logic one indicates that the PLDA EZDMA2 core is supplying data from a DMA read request.
	dma_wrchannel		: in std_logic;							-- Logic one indicates that data is available from this DMA channel.
	dma_rd				: in std_logic;							-- Logic one indicates that the PLDA EZDMA2 core is consuming data from a DMA write request.
	dma_rdchannel		: in std_logic;							-- Logic one indicates that data is needed on this DMA channel.
	dma_regin			: out std_logic_vector(127 downto 0);	-- Configuraton settings for the PLDA EZDMA2 core. 
	dma_param 			: out std_logic_vector(23 downto 0);	-- DMA parameter settings for the PLDA EZDMA2 core.
	dma_control			: out std_logic_vector(5 downto 0);		-- DMA control settings for the PLDA EZDMA2 core.
	dma_status	 		: in std_logic;							-- DMA status from the PLDA EZDMA2 core.
	dma_engine			: out std_logic_vector(2 downto 0);		-- Status of DMA engine busy (2), go (1), and enable (0) bits.
	dma_fifocnt    		: out std_logic_vector(12 downto 0)		-- FIFO count into the PLDA EZDMA2 core (not used).
);
end rx_datapath;

architecture structural of rx_datapath is

	signal dma_enable 		: std_logic_vector(1 downto 0);	-- Logic one enables the DMA on Demand controller (0= source, 1 = destination).
	signal dma_go			: std_logic_vector(1 downto 0);	-- Logic one indicates a burst transfer can be processed by the application logic (0= source, 1 = destination).
	signal dma_busy			: std_logic;					-- Logic one indicates that the DMA channel is currently active.
	signal stop_on_count	: std_logic;					-- Logic one throttles the data after the requested number of event samples pass.
	signal count_done		: std_logic;					-- Logic one indicates that the requested number of event samples have passed.
	signal channel_active	: std_logic_vector(1 downto 0);	-- Active high enable for the channel datapath (0= source, 1 = destination).
	signal payload_data		: std_logic_vector(63 downto 0);-- Payload consisting of eight 16-bit data items.
	signal payload_valid	: std_logic_vector((AXI_WIDTH/64)-1 downto 0);	-- Active high payload data valid flag.
	signal dma_fifo_rd		: std_logic;					-- Logic one indicates that the PLDA EZDMA2 core is consuming data from a DMA write request.
	signal clear			: std_logic;					-- Active high strobe to clear residue from the datapath.
	
begin

	-- Synchronizer to control the operation of the datapath.
	SYNC_DATAPATH : entity work.synchronizer
	port map (		
		clk					=> user_clk,
		sample_clk			=> rx_clk,
		rst  				=> rst,
		synchronizer_status	=> synchronizer_status,
		tx_rx_select		=> '1',
		synchronizer_on_off	=> synchronizer_on_off,
		init_status			=> '1',
		dma_enable			=> dma_enable(0),
		dma_go				=> dma_go(0),
		dma_busy			=> dma_busy,
		stop_on_count		=> stop_on_count,
		count_done			=> count_done,
		prime_done			=> '1',
		channel_active		=> channel_active(0),
		clear				=> clear,
		start_event			=> start_event,
		stop_event			=> stop_event,
		soft_trigger		=> soft_trigger,
		hard_trigger		=> hard_trigger
	);	
	
	-- Data formatter and throttle.
	FLOW_CONTROL : entity work.rx_flow_control
	port map (		
		rst  			=> rst,
		rx_clk			=> rx_clk,
		enable  		=> channel_active(1),
		data_item		=> data_item,
		payload_data	=> payload_data,
		payload_valid	=> payload_valid,
		sample_count	=> sample_count,
		stop_on_count	=> stop_on_count,
		count_done		=> count_done
	);
	
	-- FIFO to rate buffer data from the receive channel to the corresponding DMA channel.
    RX_TO_DMA : entity work.dma_write_fifo
	port map (
		clear				=> clear,
		clk					=> user_clk,
		rx_clk				=> rx_clk,
		data_in				=> payload_data,
		enable_64bit		=> payload_valid,
		fifo_error			=> fifo_error,
		dma_enable			=> dma_enable(0),
		dma_fifo_rd			=> dma_fifo_rd,
		dma_rddata			=> dma_rddata,
		dma_burst_size		=> dma_descriptor(29 downto 16),
		dma_go				=> dma_go(0)
	);
	
	-- "DMA on demand" controller assigned to the RX channel.
	ADC_DMA : entity work.dma_on_demand
	port map(
	    rst 				=> rst,
		clk 				=> user_clk,
		dma_init			=> '1',
		dma_rw				=> '1',	--	Logic zero selects a burst read operation, logic one selects write.
		dma_latency			=> "00",
		dma_enable			=> dma_enable(1),
	    dma_go				=> dma_go(1),
		dma_abort			=> '0',
	    dma_current_page	=> dma_current_page,
	    dma_current_burst	=> dma_current_burst,
	    dma_page_address	=> dma_page_address,
	    dma_descriptor		=> dma_descriptor,
		dma_fifo_wr			=> open,
		dma_fifo_rd			=> dma_fifo_rd,
		dma_marker			=> dma_marker,
		dma_busy			=> dma_busy,
		dma_wr 				=> dma_wr,
		dma_wrchannel 		=> dma_wrchannel,  
		dma_rd 				=> dma_rd,  
		dma_rdchannel 		=> dma_rdchannel,  
		dma_regin 			=> dma_regin,  
		dma_param  			=> dma_param,  	
		dma_control	 		=> dma_control,
		dma_status  		=> dma_status, 
		dma_fifocnt  		=> dma_fifocnt
	);
	
	-- Route DMA status to the BAR0 decoder.
	dma_engine <= dma_busy & dma_go(1) & dma_enable(1);
	
	-- Force continuous DMA transactions and block the channel enable during PCI bus benchmarking.
	dma_go(1) <= dma_go(0) or pci_measurement;
	dma_enable(1) <= dma_enable(0) or pci_measurement;
	channel_active(1) <= channel_active(0) and not(pci_measurement);

end structural;
